"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    createErrorLink: function() {
        return createErrorLink;
    },
    createLoadingLink: function() {
        return createLoadingLink;
    },
    createApolloClient: function() {
        return createApolloClient;
    },
    createGraphQLErrorMessage: function() {
        return createGraphQLErrorMessage;
    },
    createMocks: function() {
        return createMocks;
    }
});
const _mock = require("@graphql-tools/mock");
const _testing = require("@apollo/client/testing");
const _client = require("@apollo/client");
const _graphql = require("graphql");
function delay(ms) {
    return new Promise((resolve)=>{
        setTimeout(()=>{
            resolve();
        }, ms);
    });
}
function createMockLink(schema, rootValue = {}, context = {}, options = {}) {
    const delayMs = options?.delay ?? 0;
    return new _client.ApolloLink((operation)=>{
        return new _client.Observable((observer)=>{
            const { query, operationName, variables } = operation;
            delay(delayMs).then(()=>{
                return (0, _graphql.graphql)({
                    schema,
                    source: (0, _graphql.print)(query),
                    rootValue,
                    contextValue: context,
                    variableValues: variables,
                    operationName
                });
            }).then((result)=>{
                const originalError = result?.errors?.[0].originalError;
                if (originalError) {
                    // @ts-ignore
                    const graphQLErrors = originalError?.graphQLErrors;
                    if (graphQLErrors) {
                        observer.next({
                            errors: graphQLErrors
                        });
                    }
                    // @ts-ignore
                    const networkError = originalError.networkError;
                    if (networkError) {
                        observer.error(networkError);
                    }
                } else {
                    observer.next(result);
                }
                observer.complete();
            }).catch(observer.error.bind(observer));
        });
    });
}
function createErrorLink(graphQLError) {
    return new _client.ApolloLink(()=>{
        return new _client.Observable((observer)=>{
            delay(100).then(()=>{
                observer.next({
                    errors: createGraphQLErrorMessage(graphQLError)
                });
                observer.complete();
            }).catch(observer.error.bind(observer));
        });
    });
}
function createLoadingLink() {
    return new _client.ApolloLink(()=>{
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        return new _client.Observable(()=>{});
    });
}
function createApolloClient({ mocks, delay = 0, cacheOptions = {}, clientOptions = {}, links = ()=>{
    return [];
}, addTypename = false }) {
    let mockLink;
    if (!Array.isArray(mocks)) {
        const apolloLinkOptions = {};
        const { resolvers, introspectionResult, rootValue, context } = mocks;
        const schema = (0, _graphql.buildClientSchema)(introspectionResult);
        // @ts-ignore
        const schemaWithMocks = (0, _mock.addMocksToSchema)({
            schema,
            resolvers
        });
        if (delay) {
            apolloLinkOptions.delay = delay;
        }
        mockLink = createMockLink(schemaWithMocks, rootValue, context, apolloLinkOptions);
    } else {
        mockLink = new _testing.MockLink(mocks, addTypename);
    }
    const cache = new _client.InMemoryCache({
        ...cacheOptions,
        addTypename
    });
    return new _client.ApolloClient({
        // @ts-ignore
        cache,
        link: _client.ApolloLink.from([
            ...links(cache),
            mockLink
        ]),
        ...clientOptions
    });
}
function createGraphQLErrorMessage(graphQLError) {
    if (graphQLError) {
        return typeof graphQLError === 'string' ? [
            new _graphql.GraphQLError(graphQLError)
        ] : graphQLError;
    }
    return [
        new _graphql.GraphQLError('Unspecified error from ErrorProvider.')
    ];
}
function createMocks(query, { variables, data, newData, graphqlErrors, error, delay = 0 }) {
    return [
        {
            request: {
                query,
                variables
            },
            ...!error ? {
                result: {
                    data,
                    ...graphqlErrors ? {
                        errors: createGraphQLErrorMessage(graphqlErrors)
                    } : {}
                }
            } : {},
            error,
            newData,
            delay
        }
    ];
}

//# sourceMappingURL=utils.js.map